@@ -1,10 +1,12 @@
 # Brief help module to define the exception we use for early exits.
-package Zonemaster::Exception::NormalExit;
+package Zonemaster::Engine::Exception::NormalExit;
 use Moose;
-extends 'Zonemaster::Exception';
+extends 'Zonemaster::Engine::Exception';
 
 # The actual interesting module.
-package Zonemaster::CLI v1.0.1;
+package Zonemaster::CLI;
+
+use version; our $VERSION = version->declare("v1.1.1");
 
 use 5.014002;
 use warnings;
@@ -13,20 +15,22 @@ use Locale::TextDomain 'Zonemaster-CLI';
 use Moose;
 with 'MooseX::Getopt';
 
-use Zonemaster;
-use Zonemaster::Logger::Entry;
-use Zonemaster::Translator;
-use Zonemaster::Util qw[pod_extract_for];
-use Zonemaster::Exception;
+use Zonemaster::Engine;
+use Zonemaster::Engine::Logger::Entry;
+use Zonemaster::Engine::Translator;
+use Zonemaster::Engine::Util qw[pod_extract_for];
+use Zonemaster::Engine::Exception;
+use Zonemaster::Engine::Zone;
 use Scalar::Util qw[blessed];
 use Encode;
-use Net::LDNS;
+use Zonemaster::LDNS;
 use POSIX qw[setlocale LC_MESSAGES];
 use List::Util qw[max];
 use Text::Reflow qw[reflow_string];
 use JSON::XS;
 
-our %numeric = Zonemaster::Logger::Entry->levels;
+our %numeric = Zonemaster::Engine::Logger::Entry->levels;
+our $JSON    = JSON::XS->new->allow_blessed->convert_blessed->canonical;
 
 STDOUT->autoflush( 1 );
 
@@ -61,6 +65,20 @@ has 'json' => (
     documentation => __( 'Flag indicating if output should be in JSON or not.' ),
 );
 
+has 'json_stream' => (
+    is            => 'ro',
+    isa           => 'Bool',
+    default       => 0,
+    documentation => __( 'Flag indicating if output should be streaming JSON or not.' ),
+);
+
+has 'json_translate' => (
+    is            => 'ro',
+    isa           => 'Bool',
+    default       => 0,
+    documentation => __( 'Flag indicating if streaming JSON output should include the translated message of the tag or not.' ),
+);
+
 has 'raw' => (
     is            => 'ro',
     isa           => 'Bool',
@@ -92,7 +110,7 @@ has 'show_module' => (
 has 'ns' => (
     is            => 'ro',
     isa           => 'ArrayRef',
-    documentation => __( 'A name/ip string giving a nameserver for undelegated tests. Can be given multiple times.' ),
+    documentation => __( 'A name/ip string giving a nameserver for undelegated tests, or just a name which will be looked up for IP addresses. Can be given multiple times.' ),
 );
 
 has 'save' => (
@@ -223,9 +241,8 @@ has 'dump_policy' => (
 
 has 'sourceaddr' => (
     is => 'ro',
-    isa => 'Maybe[Str]',
+    isa => 'Str',
     required => 0,
-    default => undef,
     documentation => __( 'Local IP address that the test engine should try to send its requests from.' ),
 );
 
@@ -260,21 +277,21 @@ sub run {
         print_test_list();
     }
 
-    Zonemaster->config->ipv4_ok(0+$self->ipv4);
-    Zonemaster->config->ipv6_ok(0+$self->ipv6);
+    Zonemaster::Engine->config->ipv4_ok(0+$self->ipv4);
+    Zonemaster::Engine->config->ipv6_ok(0+$self->ipv6);
 
     if ($self->sourceaddr) {
-        Zonemaster->config->resolver_source($self->sourceaddr);
+        Zonemaster::Engine->config->resolver_source($self->sourceaddr);
     }
 
     if ( $self->policy ) {
         say __( "Loading policy from " ) . $self->policy . '.' if not ($self->dump_config or $self->dump_policy);
-        Zonemaster->config->load_policy_file( $self->policy );
+        Zonemaster::Engine->config->load_policy_file( $self->policy );
     }
 
     if ( $self->config ) {
         say __( "Loading configuration from " ) . $self->config . '.' if not ($self->dump_config or $self->dump_policy);
-        Zonemaster->config->load_config_file( $self->config );
+        Zonemaster::Engine->config->load_config_file( $self->config );
     }
 
     if ( $self->dump_config ) {
@@ -282,8 +299,8 @@ sub run {
     }
 
     if ( $self->dump_policy ) {
-        foreach my $mod (Zonemaster->modules) {
-            Zonemaster->config->load_module_policy($mod)
+        foreach my $mod (Zonemaster::Engine->modules) {
+            Zonemaster::Engine->config->load_module_policy($mod)
         }
         do_dump_policy();
     }
@@ -297,20 +314,27 @@ sub run {
     }
 
     my $translator;
-    $translator = Zonemaster::Translator->new unless ( $self->raw or $self->json );
+    $translator = Zonemaster::Engine::Translator->new unless ( $self->raw or $self->json or $self->json_stream );
     $translator->locale( $self->locale ) if $translator and $self->locale;
     eval { $translator->data } if $translator;    # Provoke lazy loading of translation data
 
+    my $json_translator;
+    if ( $self->json_translate ) {
+        $json_translator = Zonemaster::Engine::Translator->new;
+        $json_translator->locale( $self->locale ) if $self->locale;
+        eval { $json_translator->data };
+    }
+
     if ( $self->restore ) {
-        Zonemaster->preload_cache( $self->restore );
+        Zonemaster::Engine->preload_cache( $self->restore );
     }
 
     # Callback defined here so it closes over the setup above.
-    Zonemaster->logger->callback(
+    Zonemaster::Engine->logger->callback(
         sub {
             my ( $entry ) = @_;
 
-            $self->print_spinner();
+            $self->print_spinner() unless $self->json_stream;
 
             $counter{ uc $entry->level } += 1;
 
@@ -332,6 +356,18 @@ sub run {
 
                     say $translator->translate_tag( $entry );
                 }
+                elsif ( $self->json_stream ) {
+                    my %r;
+
+                    $r{timestamp} = $entry->timestamp;
+                    $r{module}    = $entry->module;
+                    $r{tag}       = $entry->tag;
+                    $r{level}     = $entry->level;
+                    $r{args}      = $entry->args if $entry->args;
+                    $r{message}   = $json_translator->translate_tag( $entry ) if $json_translator;
+
+                    say $JSON->encode( \%r );
+                }
                 elsif ( $self->json ) {
                     # Don't do anything
                 }
@@ -343,7 +379,7 @@ sub run {
                 }
             } ## end if ( $numeric{ uc $entry...})
             if ( $self->stop_level and $numeric{ uc $entry->level } >= $numeric{ uc $self->stop_level } ) {
-                die( Zonemaster::Exception::NormalExit->new( { message => "Saw message at level " . $entry->level } ) );
+                die( Zonemaster::Engine::Exception::NormalExit->new( { message => "Saw message at level " . $entry->level } ) );
             }
         }
     );
@@ -397,15 +433,15 @@ sub run {
             foreach my $t ( @{ $self->test } ) {
                 my ( $module, $method ) = split( '/', $t, 2 );
                 if ( $method ) {
-                    Zonemaster->test_method( $module, $method, Zonemaster->zone( $domain ) );
+                    Zonemaster::Engine->test_method( $module, $method, Zonemaster::Engine->zone( $domain ) );
                 }
                 else {
-                    Zonemaster->test_module( $module, $domain );
+                    Zonemaster::Engine->test_module( $module, $domain );
                 }
             }
         }
         else {
-            Zonemaster->test_zone( $domain );
+            Zonemaster::Engine->test_zone( $domain );
         }
     };
     if ( $translator ) {
@@ -416,7 +452,7 @@ sub run {
 
     if ( $@ ) {
         my $err = $@;
-        if ( blessed $err and $err->isa( "Zonemaster::Exception::NormalExit" ) ) {
+        if ( blessed $err and $err->isa( "Zonemaster::Engine::Exception::NormalExit" ) ) {
             say STDERR "Exited early: " . $err->message;
         }
         else {
@@ -433,7 +469,7 @@ sub run {
     }
 
     if ( $self->nstimes ) {
-        my $zone = Zonemaster->zone( $domain );
+        my $zone = Zonemaster::Engine->zone( $domain );
         my $max = max map { length( "$_" ) } @{ $zone->ns };
 
         print "\n";
@@ -452,16 +488,16 @@ sub run {
     }
 
     if ($self->elapsed) {
-        my $last = Zonemaster->logger->entries->[-1];
+        my $last = Zonemaster::Engine->logger->entries->[-1];
         printf "Total test run time: %0.1f seconds.\n", $last->timestamp;
     }
 
     if ( $self->json ) {
-        say Zonemaster->logger->json( $self->level );
+        say Zonemaster::Engine->logger->json( $self->level );
     }
 
     if ( $self->save ) {
-        Zonemaster->save_cache( $self->save );
+        Zonemaster::Engine->save_cache( $self->save );
     }
 
     return;
@@ -469,20 +505,31 @@ sub run {
 
 sub add_fake_delegation {
     my ( $self, $domain ) = @_;
+    my @ns_with_no_ip;
     my %data;
 
     foreach my $pair ( @{ $self->ns } ) {
         my ( $name, $ip ) = split( '/', $pair, 2 );
 
-        if ( not $name or not $ip ) {
-            say STDERR "Malformed --ns switch, need name and IP separated by a /.";
+        if ( not $name ) {
+            say STDERR "--ns must have be a name or a name/ip pair.";
             exit( 1 );
         }
 
-        push @{ $data{ $self->to_idn( $name ) } }, $ip;
+        if ($ip) {
+            push @{ $data{ $self->to_idn( $name ) } }, $ip;
+        }
+        else {
+            push @ns_with_no_ip, $self->to_idn($name);
+        }
+    }
+    foreach my $ns ( @ns_with_no_ip ) {
+        if ( not exists $data{ $ns } ) {
+            $data{ $ns } = undef;
+        }
     }
 
-    Zonemaster->add_fake_delegation( $domain => \%data );
+    Zonemaster::Engine->add_fake_delegation( $domain => \%data );
 
     return;
 }
@@ -496,19 +543,19 @@ sub add_fake_ds {
         push @data, { keytag => $tag, algorithm => $algo, type => $type, digest => $digest };
     }
 
-    Zonemaster->add_fake_ds( $domain => \@data );
+    Zonemaster::Engine->add_fake_ds( $domain => \@data );
 
     return;
 }
 
 sub print_versions {
     say 'CLI version:    ' . __PACKAGE__->VERSION;
-    say 'Engine version: ' . $Zonemaster::VERSION;
+    say 'Engine version: ' . $Zonemaster::Engine::VERSION;
     say "\nTest module versions:";
 
-    my %methods = Zonemaster->all_methods;
+    my %methods = Zonemaster::Engine->all_methods;
     foreach my $module ( sort keys %methods ) {
-        my $mod = "Zonemaster::Test::$module";
+        my $mod = "Zonemaster::Engine::Test::$module";
         say "\t$module: " . $mod->version;
     }
 
@@ -534,17 +581,17 @@ sub to_idn {
         return $str;
     }
 
-    if ( Net::LDNS::has_idn() ) {
-        return Net::LDNS::to_idn( encode( 'utf8', decode( $self->encoding, $str ) ) );
+    if ( Zonemaster::LDNS::has_idn() ) {
+        return Zonemaster::LDNS::to_idn( decode( $self->encoding, $str ) );
     }
     else {
-        say STDERR __( "Warning: Net::LDNS not compiled with libidn, cannot handle non-ASCII names correctly." );
+        say STDERR __( "Warning: Zonemaster::LDNS not compiled with libidn, cannot handle non-ASCII names correctly." );
         return $str;
     }
 }
 
 sub print_test_list {
-    my %methods = Zonemaster->all_methods;
+    my %methods = Zonemaster::Engine->all_methods;
     my $maxlen  = max map {
         map { length( $_ ) }
           @$_
@@ -573,13 +620,13 @@ sub print_test_list {
 
 sub do_dump_policy {
     my $json = JSON::XS->new->canonical->pretty;
-    print $json->encode(Zonemaster->config->policy);
+    print $json->encode(Zonemaster::Engine->config->policy);
     exit;
 }
 
 sub do_dump_config {
     my $json = JSON::XS->new->canonical->pretty;
-    print $json->encode(Zonemaster->config->get);
+    print $json->encode(Zonemaster::Engine->config->get);
     exit;
 }
 
@@ -593,3 +640,11 @@ __END__
 =head1 NAME
 
 Zonemaster::CLI - run Zonemaster tests from the command line
+
+=head1 AUTHORS
+
+Vincent Levigneron <vincent.levigneron at nic.fr>
+- Current maintainer
+
+Calle Dybedahl <calle at init.se>
+- Original author
